package cn.estsh.i3plus.core.apiservice.serviceimpl.busi;

import cn.estsh.i3plus.core.api.iservice.busi.ILogSystemService;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.common.PagerHelper;
import cn.estsh.i3plus.pojo.platform.bean.LogSystem;
import cn.estsh.i3plus.pojo.platform.repositorymongo.LogSystemRepository;
import cn.estsh.i3plus.pojo.platform.sqlpack.CoreBsonPack;
import org.bson.conversions.Bson;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @Description : 系统日志服务接口实现
 * @Reference :
 * @Author : yunhao
 * @Date : 2018-10-25 10:41
 * @Modify :
 **/
@Service
public class LogSystemService implements ILogSystemService {
    public static final Logger LOGGER = LoggerFactory.getLogger(LogSystemService.class);

    @Autowired
    private LogSystemRepository logSystemRDao;

    @Override
    public void insertLogSystem(LogSystem logSystem) {
        LOGGER.info("系统日志 SYS_SYSTEM {}",logSystem);
        logSystemRDao.insert(logSystem);
    }

    @Override
    public void deleteLogSystemById(String id) {
        LOGGER.info("系统日志 SYS_SYSTEM id:{}",id);
        logSystemRDao.deleteById(Long.parseLong(id));
    }

    @Override
    public List<LogSystem> listLogSystem() {
        LOGGER.info("系统日志 LOG_SYSTEM list");
        return logSystemRDao.findAll();
    }

    @Override
    public LogSystem getLogSystemById(long id) {
        LOGGER.info("系统日志 LOG_SYSTEM id:{}",id);
        return logSystemRDao.getById(id);
    }

    @Override
    public ListPager queryLogSystemByPager(LogSystem logSystem, Pager pager) {
        LOGGER.info("系统日志 LOG_SYSTEM logSystem:{}，Pager:{}",logSystem,pager);
        if (logSystem == null) {
            pager = PagerHelper.getPager(pager,(int) logSystemRDao.listCount());
            return new ListPager(logSystemRDao.listPager(pager),pager);
        } else {
            Bson bson = CoreBsonPack.packBsonByLogSystem(logSystem);
            pager = PagerHelper.getPager(pager, (int) logSystemRDao.findByBsonCount(bson));
            return new ListPager(logSystemRDao.findByBsonPager(bson,pager,logSystem.getOrderByParam()
                                    ,logSystem.getAscOrDesc()),pager);
        }
    }
}
