package cn.estsh.i3plus.core.apiservice.serviceimpl.busi;

import cn.estsh.i3plus.core.api.iservice.busi.IOrganizeService;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.common.PagerHelper;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.HqlPack;
import cn.estsh.i3plus.pojo.platform.bean.Organize;
import cn.estsh.i3plus.pojo.platform.bean.Position;
import cn.estsh.i3plus.pojo.platform.bean.SessionUser;
import cn.estsh.i3plus.pojo.platform.repository.OrganizeRepository;
import cn.estsh.i3plus.pojo.platform.sqlpack.CoreHqlPack;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.exception.ImppExceptionEnum;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @Description : 组织业务接口实现 
 * @Reference   :
 * @Author      : wei.peng
 * @Date        : 2018-10-22 16:58:42.957
 * @Modify      :
 **/
@Service
public class OrganizeService implements IOrganizeService {

    public static final Logger LOGGER = LoggerFactory.getLogger(OrganizeService.class);

    @Autowired
    private OrganizeRepository organizeRDao;

    @Autowired
    private CoreTreeService coreTreeService;

    @Override
    public void updateOrganize(Organize organize) {
        LOGGER.info("组织 ORGANIZE :{}", organize);
        // 唯一校验 CODE 相同 ID 不同 则视为相同
        List<Organize> list = organizeRDao.findByProperty("organizeCode", organize.getOrganizeCode());
        if(null != list && list.size() > 0){
            for (Organize org : list) {
                if(!org.getId().equals(organize.getId())){
                    throw ImppExceptionBuilder.newInstance()
                            .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                            .setErrorCode(ImppExceptionEnum.IO_EXCEPTION.getCode())
                            .setErrorDetail("组织代码已存在")
                            .setErrorSolution("请重新输入组织代码")
                            .build();
                }
            }
        }

        // 父节点检查
        coreTreeService.doValidatorOrganizeParentId(organize);

        // 冗余父节点
        if(organize.getParentId() != null && organize.getParentId() > 0){
            Organize org = getOrganizeById(organize.getParentId().toString());
            ValidatorBean.checkNotNull(org,"上级组织信息不存在");

            organize.setRedParentName(org.getName());
        }

        // 更新数据
        organizeRDao.update(organize);
        // 修改冗余信息
        organizeRDao.updateByProperties("parentId",organize.getId(),
                "redParentName",organize.getName());
    }

    @Override
    public void updateOrganizeStatus(String id, int status, SessionUser user) {
        LOGGER.info("组织 ORGANIZE  id:{} status:{} user:{}", id,status,user);
        Organize organize = organizeRDao.getById(Long.parseLong(id));
        organize.setOrganizeStatusId(status);
        organize.setModifyUser(user.getUserName());
        organizeRDao.update(organize);
    }

    @Override
    public void deleteOrganizeById(String id) {
        LOGGER.info("组织 ORGANIZE Key:{}", id);
        long count = organizeRDao.findByPropertyCount("parentId", Long.parseLong(id));
        if (count >= 1) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.IMPP.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION_DATA_NOT_EXIT.getCode())
                    .setErrorDetail("存在相关组织信息无法删除！")
                    .setErrorSolution("请先删除子集信息在操作")
                    .build();
        }
        organizeRDao.deleteById(Long.parseLong(id));
    }

    @Override
    public void deleteOrganizeByIds(String[] ids) {
        long positionCount = organizeRDao.findByHqlWhereCount(CoreHqlPack.packHqlIds("parentId",ids));
        if (positionCount >= 1) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("存在相关组织信息无法删除！")
                    .setErrorSolution("请先删除子集信息在操作")
                    .build();
        }else {
            Long[] array = new Long[ids.length];
            for (int i = 0; i < ids.length; i++) {
                array[i] = Long.parseLong(ids[i]);
            }
            organizeRDao.deleteByIds(array);
        }
    }

    @Override
    public Organize insertOrganize(Organize organize) {
        LOGGER.info("组织 ORGANIZE :{}", organize);

        // 代码唯一校验
        long count = organizeRDao.findByPropertyCount("organizeCode", organize.getOrganizeCode());
        if(count <= 0){
            // 冗余信息
            if(organize.getParentId() != null && organize.getParentId() > 0){
                Organize org = getOrganizeById(organize.getParentId().toString());
                ValidatorBean.checkNotNull(org,"上级组织信息不存在");

                organize.setRedParentName(org.getName());
            }

            // 插入数据入库
            return organizeRDao.insert(organize);
        } else {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                    .setErrorCode(ImppExceptionEnum.IO_EXCEPTION.getCode())
                    .setErrorDetail("组织代码已存在")
                    .setErrorSolution("请重新输入组织代码")
                    .build();
        }
    }

    @Override
    public List<Organize> listOrganize() {
        LOGGER.info("组织 ORGANIZE find All");
        return organizeRDao.findAll();
    }

    @Override
    public ListPager<Organize> queryOrganize(Organize organize, Pager pager) {
        LOGGER.info("组织信息 Organize find Organize page");

        if(organize == null){
            //不传入实体对象，查询所有
            int count = organizeRDao.listCount();
            pager = PagerHelper.getPager(pager,count);
            return new ListPager(organizeRDao.listPager(pager),pager);
        }else {
            //生成hql查询语句
            String hqlPack = CoreHqlPack.packHqlOrganize(organize);
            pager = PagerHelper.getPager(pager,organizeRDao.findByHqlWhereCount(hqlPack));
            return new ListPager(organizeRDao.findByHqlWherePage(hqlPack + organize.orderBy(), pager),pager);
        }
    }

    @Override
    public Organize getOrganizeById(String id) {
        LOGGER.info("组织 ORGANIZE find id:{}", id);
        return organizeRDao.getById(Long.parseLong(id));
    }
}
