package cn.estsh.i3plus.core.apiservice.serviceimpl.busi;

import cn.estsh.i3plus.core.api.iservice.busi.ISysUserService;
import cn.estsh.i3plus.core.apiservice.dao.IUserPermissionDao;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.tool.EncryptTool;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.common.PagerHelper;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.shirotoken.AdminToken;
import cn.estsh.i3plus.pojo.base.shirotoken.SaAdminToken;
import cn.estsh.i3plus.pojo.base.shirotoken.UserToken;
import cn.estsh.i3plus.pojo.platform.bean.*;
import cn.estsh.i3plus.pojo.platform.repository.*;
import cn.estsh.i3plus.pojo.platform.sqlpack.CoreHqlPack;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.exception.ImppExceptionEnum;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import org.apache.shiro.authc.AuthenticationToken;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

/**
 * @Description : 平台用户业务
 * @Reference :
 * @Author : wei.peng
 * @Date : 2018-10-25 17:45
 * @Modify :
 **/
@Service
public class SysUserService implements ISysUserService {

    public static final Logger LOGGER = LoggerFactory.getLogger(SysUserService.class);

    @Autowired
    private SysUserRepository sysUserRDao;

    @Autowired
    private SysUserInfoRepository sysUserInfoRDao;

    @Autowired
    private IUserPermissionDao userPermissionDao;

    @Autowired
    private OrganizeRepository organizeRDao;

    @Autowired
    private SysRoleRepository sysRoleRDao;

    @Autowired
    private DepartmentRepository departmentRDao;

    @Autowired
    private PositionRepository positionRDao;

    @Autowired
    private RefUserRoleRepository refUserRoleRDao;

    @Autowired
    private RefUserDepartmentRepository refUserDepartmentRDao;

    @Autowired
    private RefUserPositionRepository refUserPositionRDao;

    @Override
    public SessionUser queryUserLogin(String loginName, String password, String languageCode) {
        LOGGER.debug("平台用户 SYS_USER loginName:{} \t password:{} \t languageCode:{}", loginName, password, languageCode);

        SysUser user = getSysUserByLoginName(loginName);

        if(user == null){
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                    .setErrorCode(ImppExceptionEnum.LOGIN_USER_NAME_EXCEPTION.getCode())
                    .setErrorDetail("用户名不存在")
                    .setErrorSolution("请重新输入用户名")
                    .build();
        }

        AuthenticationToken token = null;
        switch (user.getUserTypeId()) {
            case 2:
                token = new AdminToken(loginName, password, languageCode);
                break;
            case 3:
                token = new SaAdminToken(loginName, password, languageCode);
                break;
            default:
                token = new UserToken(loginName, password, languageCode);
                break;
        }
        SessionUser sessionUser = AuthUtil.login(token);

        packSessionUser(sessionUser, user, user.getUserTypeId(), languageCode);
        AuthUtil.setSessionUser(sessionUser);
        return sessionUser;
    }

    @Override
    public void updateSysUserStatus(String id, int status, SessionUser user) {
        LOGGER.debug("平台用户 SYS_USER id:{} status:{} modifyUser:{}", id, status, user.getUserName());
        SysUser sysUser = sysUserRDao.getById(Long.parseLong(id));
//        sysUser.setUserStatusId(status);
        sysUser.setModifyUser(user.getUserName());
        sysUserRDao.update(sysUser);
    }

    @Override
    public void updateBatchSysUserStatus(String[] ids, int status, SessionUser user) {
        LOGGER.debug("平台用户 SYS_USER DELETE By ids :{} status:{}, SessionUser :{}", ids,status,user);

        String where = CoreHqlPack.packHqlIds("id", ids);
        List<SysUser> list = sysUserRDao.findByHqlWhere(where);
        if(null != list && list.size() > 0){
            for (SysUser sysUser : list) {
//                sysUser.setUserStatusId(status);
                sysUser.setIsValid(CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue());
                ConvertBean.modelUpdate(sysUser,user);
            }
            sysUserRDao.saveAll(list);
        }
    }

    @Override
    public void updateSysUser(SysUser sysUser) throws Exception{
        LOGGER.debug("平台用户 SYS_USER SysUser :{}", sysUser);

        SysUser loginUser = getSysUserByLoginName(sysUser.getUserLoginName());
        // 判断是否修改登录名称
        if (loginUser == null || loginUser.getId().equals(sysUser.getId())) {
            // 密码加密 输入的密码不等于密码一致时修改密码
            if(null != sysUser.getUserLoginPassword() && sysUser.getUserLoginPassword().trim().length() > 0){
                if(sysUser.getUserLoginPassword().equals(loginUser.getUserLoginPassword())){
                    String md5 = EncryptTool.hexMD5(sysUser.getUserLoginPassword());
                    sysUser.setUserLoginPassword(md5);
                }
            }

            sysUserRDao.save(sysUser);
        }else {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION_DATA_EXIT.getCode())
                    .setErrorDetail("用户名已存在")
                    .setErrorSolution("请重新输入用户名")
                    .build();
        }
    }

    @Override
    public void updateSysUserPassword(String userIdStr, String password, String newPwd) throws Exception {
        LOGGER.debug("平台用户 SYS_USER userIdStr:{} password:{} newPwd:{}", userIdStr,password,newPwd);
        SysUser user = sysUserRDao.getById(Long.parseLong(userIdStr));
        if(user != null){
            String md5 = EncryptTool.hexMD5(password);
            if(md5.equals(user.getUserLoginPassword())){
                user.setUserLoginPassword(EncryptTool.hexMD5(newPwd));
                sysUserRDao.save(user);
            }else {
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                        .setErrorCode(ImppExceptionEnum.LOGIN_USER_NAME_PASSWORD_UPDATE_EXCEPTION.getCode())
                        .setErrorDetail("旧密码错误")
                        .setErrorSolution("请重新输入")
                        .build();
            }
        }else {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION_DATA_NOT_EXIT.getCode())
                    .setErrorDetail("用户不存在")
                    .setErrorSolution("请重新操作")
                    .build();
        }
    }

    @Override
    public SysUser insertSysUser(SysUser sysUser)throws Exception {
        LOGGER.debug("平台用户 SYS_USER SysUser:{}", sysUser);

        SysUser loginUser = getSysUserByLoginName(sysUser.getUserLoginName());
        if (null == loginUser) {
            // 密码加密 输入的密码不等于密码一致时修改密码
            String md5 = EncryptTool.hexMD5(sysUser.getUserLoginPassword());
            sysUser.setUserLoginPassword(md5);
            return sysUserRDao.insert(sysUser);
        } else {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                    .setErrorCode(ImppExceptionEnum.IO_EXCEPTION.getCode())
                    .setErrorDetail("用户名已存在")
                    .setErrorSolution("请重新输入用户名")
                    .build();
        }
    }

    @Override
    public void refreshRefUserRole(SysUser sysUser, String[] roleIds, SessionUser user) {
        LOGGER.debug("平台用户 SYS_USER SysUser:{}", sysUser);
        LOGGER.debug("平台用户 SYS_USER String[] :{}", roleIds);

        // 根据IDS 查询所有的角色信息
        String where = CoreHqlPack.packHqlIds("id", roleIds);
        List<SysRole> list = sysRoleRDao.findByHqlWhere(where);

        // 删除角色关系信息
        refUserRoleRDao.deleteByProperty("userId", sysUser.getId()); // 删除用户角色关系

        if(null != list && list.size() > 0){
            // 新增用户角色关系
            List<RefUserRole> refUserRoles = new ArrayList<>();
            List<String> roleNameList = new ArrayList<>();
            RefUserRole refUserRole = null;
            for (SysRole role : list) {
                roleNameList.add(role.getName());
                // 用户角色关系创建
                refUserRole = new RefUserRole();
                refUserRole.setRoleId(role.getId());
                refUserRole.setRoleNameRdd(role.getName());
                refUserRole.setUserId(sysUser.getId());
                refUserRole.setUserNameRdd(sysUser.getUserName());

                refUserRoles.add(refUserRole);
            }
            refUserRoleRDao.saveAll(refUserRoles);

            // 角色名称冗余
            sysUser.setRoleNameRdd(String.join(",", roleNameList));
            sysUser.setRoleId(list.get(0).getId());

            sysUserRDao.save(sysUser);
        }
    }

    @Override
    public void deleteSysUserById(String id) {
        LOGGER.debug("平台用户 SYS_USER DELETE By id :{}", id);
        sysUserRDao.updateByProperties("id", id, "isValid", CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue());

        // 删除用户关系(物理删除)
        refUserRoleRDao.deleteByProperty("userId", Long.parseLong(id)); // 删除用户角色关系
        refUserDepartmentRDao.deleteByProperty("userId", Long.parseLong(id)); //删除用户部门关系
        refUserPositionRDao.deleteByProperty("userId", Long.parseLong(id)); //删除用户部门关系
    }

    @Override
    public void deleteBatchSysUserById(String[] ids,SessionUser user) {
        LOGGER.debug("平台用户 SYS_USER DELETE By ids :{}", ids);

        // 标记用户已删除
        String where = CoreHqlPack.packHqlIds("id", ids);
        List<SysUser> list = sysUserRDao.findByHqlWhere(where);
        if(null != list && list.size() > 0){
            for (SysUser sysUser : list) {
                sysUser.setIsValid(CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue());
            }

            sysUserRDao.saveAll(list);
            refUserRoleRDao.deleteByPropertyIn("userId",ids);
        }
    }

    @Override
    public List<SysUser> list() {
        LOGGER.debug("平台用户 SYS_USER find All");
        return sysUserRDao.findByProperty("isValid", CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue());
    }

    @Override
    public ListPager<SysUser> querySysUserByPager(SysUser user, Pager pager) {
        LOGGER.debug("系统用户 SysUser find user :{} page :{}", user, pager);

        //生成hql查询语句
        String hqlPack = CoreHqlPack.packHqlSysUser(user);
        pager = PagerHelper.getPager(pager, sysUserRDao.findByHqlWhereCount(hqlPack));
        return new ListPager(sysUserRDao.findByHqlWherePage(hqlPack + user.orderBy(), pager), pager);
    }

    @Override
    public SysUser getSysUserById(String id) {
        LOGGER.debug("平台用户 SYS_USER get By id :{}", id);
        return sysUserRDao.getById(Long.parseLong(id));
    }

    @Override
    public SysUserInfo getSysUserInfoById(Long id) {
        return sysUserInfoRDao.getByProperty("id",id);
    }

    @Override
    public SysUser getSysUserByLoginName(String loginName) {
        LOGGER.debug("平台用户 SYS_USER find By Login Name :{}", loginName);
        return sysUserRDao.getByProperty(new String[]{"userLoginName", "isValid"},
                new Object[]{loginName, CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue()});
    }

    @Override
    public List<RefUserRole> findRefUserRoleByUserId(String userId) {
        return refUserRoleRDao.findByProperty("userId",Long.parseLong(userId));
    }

    /**
     * 封装用户登陆信息
     *
     * @param sessionUser  当前会话信息
     * @param user         当前登录用户
     * @param userType     用户类型
     * @param languageCode 用户选择的语言
     * @return
     */
    public SessionUser packSessionUser(SessionUser sessionUser, SysUser user, Integer userType, String languageCode) {

        LOGGER.debug("平台用户 SYS_USER user id :{}", user.getId());
        SysUserInfo userInfo = sysUserInfoRDao.getById(user.getUserInfoId());
        Organize organize = organizeRDao.getById(userInfo.getOrganizeId());

        // 权限信息查询
        List<SysMenu> menuList = userPermissionDao.findSysMenuByUser(user);
        List<SysRole> roleList = userPermissionDao.findSysRoleByUser(user);
        List<Department> departmentList = userPermissionDao.findDepartmentByUser(userInfo);
        List<Position> positionList = userPermissionDao.findPositionByUser(userInfo);

        LOGGER.debug("平台用户 SYS_USER userInfo :{}", userInfo);
        LOGGER.debug("平台用户 SYS_USER organize :{}", organize);
        LOGGER.debug("平台用户 SYS_USER departmentList  size :{}", departmentList.size());
        LOGGER.debug("平台用户 SYS_USER positionList size :{}", positionList.size());
        LOGGER.debug("平台用户 SYS_USER menuList size :{}", menuList.size());
        LOGGER.debug("平台用户 SYS_USER roleList size :{}", roleList.size());

        user.setMenuList(menuList);
        user.setRoleList(roleList);

        userInfo.setUser(user);
        userInfo.setOrganize(organize);
        userInfo.setDepartmentList(departmentList);
        userInfo.setPositionList(positionList);

        sessionUser.setLanguageCode(languageCode);
        sessionUser.setUserName(userInfo.getName());
        sessionUser.setUserTypeId(CommonEnumUtil.USER_TYPE.valueOf(userType));
        sessionUser.setUserCode(userInfo.getUserEmpNo());
        sessionUser.setUser(user);
        sessionUser.setUserInfo(userInfo);

        return sessionUser;
    }

}
